import requests

from gclick_sdk.gclick import DEFAULT_TIMEOUT_UPLOAD, ENDPOINT_ATIVIDADES_RESPOSTA
from gclick_sdk.atividade_exception import AtividadeNotFoundException, BaixadaAnteriormenteException, BaixaUnknowException
from gclick_sdk.credentials import GClickCredentials
from gclick_sdk.exception import UnauthorizedException


class AtividadesClient:
    """
    Client para os endpoints destinados à manipulação de Atividades.
    """

    def __init__(self, credentials: GClickCredentials):
        self._credentials = credentials

    def baixar_upload(
        self,
        id_atividade: str,
        inscricao: str,
        competencia: int,
        file_content: bytes,
        file_name: str,
        file_mimetype: str,
        usuario: str = None
    ):
        """
        Realiza a baixa de atividade realizando o upload de arquivo.

        Parâmetros:
        - id_atividade: ID da atividade.
        - inscricao: Inscrição do cliente do escritório contábil (CPF ou CNPJ).
        - competencia: Data da competência no formato AAAAMM.
        - file_content: Conteúdo, em bytes, do arquivo a sofrer upload.
        - file_name: Nome do arquivo sofrendo upload.
        - file_mimetype: Mimetype do arquivo sofrendo upload.
        - usuario [opcional]: Apelido do usuário no sistema parceiro.

        Obs.: Se o parâmetro "usuario" não for passado, será assumido o usuário recebido com o objeto
        de credenciais (no construtor da classe).
        """
        # TODO Se eu não prover um arquivo para o endpoint de baixa v3, ele faz a baixa sem o arquivo?

        # Resolvendo apelido do usuário
        if usuario is None:
            usuario = self._credentials.usuario

        # Montando corpo da requisição
        data = {
            'idAtividade': id_atividade,
            'inscricao': inscricao,
            'competencia': competencia,
            'usuario': usuario,
            'sistema': self._credentials.sistema
        }

        # Preparando o arquivo para upload
        if (
            file_content is None
            or file_content is None
            or file_mimetype is None
        ):
            raise BaixaUnknowException('File parameter missing')

        files = {
            'arquivo': (file_name, file_content, file_mimetype)
        }

        # Preparando o header de autenticação
        access_token = self._credentials.get_access_token()
        headers = {
            'Authorization': f'Bearer {access_token}'
        }

        # Realizando a requisição
        resp = requests.post(ENDPOINT_ATIVIDADES_RESPOSTA,
                             data=data, files=files, timeout=DEFAULT_TIMEOUT_UPLOAD, headers=headers)

        if resp.status_code in [200, 201]:
            return
        elif resp.status_code == 204:
            raise BaixadaAnteriormenteException(resp.text)
        elif resp.status_code == 401:
            raise UnauthorizedException(resp.text)
        elif resp.status_code == 404:
            raise AtividadeNotFoundException(resp.text)
        elif resp.status_code == 400:
            raise BaixaUnknowException(resp.text)
        else:
            resp.raise_for_status()



import base64
import base64
import requests

from gclick_sdk.exception import AuthenticationException
from gclick_sdk.gclick import DEFAULT_TIMEOUT_AUTHENTICATION, ENDPOINT_ACCESS_TOKEN


class GClickCredentials:
    """
    Classe destinada ao controle de autenticação com as APIs do GClick.
    """

    sistema: int
    usuario: str
    empresa: int
    _access_token: str
    _token_type: str
    _access_token_expires_in: int
    _access_token_scope: str
    _refresh_token: str

    def __init__(
        self,
        sistema: int,
        usuario: str,
        conta: str,
        senha: str,
        empresa: int,
        client_id: str,
        client_secret: str
    ):
        """
        Construtor padrão. Parâmetros:

        - sistema: Código do sistema informado pelo G-Click (para integração).
        - usuario: Apelido do usuário no sistema parceiro correspondente ao apelido do usuário no G-Click.
        - conta: Conta de acesso do usuário junto ao GClick
        - senha: Senha de acesso do usuário junto ao GClick
        - empresa: Código da empresa parceira provido pelo GClick, para integração.
        - client_id: ID da aplicação cliente cadastrada no servidor de autenticação.
        - client_secret: Chave da aplicação cliente cadastrada no servidor de autenticação.
        """
        self.sistema = sistema
        self.usuario = usuario
        self._conta = conta
        self._senha = senha
        self.empresa = empresa
        self._client_id = client_id
        self._client_secret = client_secret

        self._access_token = None
        self._token_type = None
        self._access_token_expires_in = None
        self._access_token_scope = None
        self._refresh_token = None

        # TODO Refatorar para autenticação inicial (sem guardar conta e senha)

    def get_access_token(self) -> str:
        """
        Retorna um access_token do usuário, já autenticado, para comunicação com as APIs do GClick.

        Exceções:
        - AuthenticationException: Representa qualquer problema de autenticação junto ao servidor de autenticação (a mensagem contém maiores detalhes).
        """
        # TODO Refatorar para controle de validação do access_token e renovação pelo refresh_token

        # Construindo a requisição
        data = {
            'username': self._conta,
            'password': self._senha,
            'grant_type': 'password',
            'empresa': self.empresa
        }

        headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Authorization': f'Basic {self._get_basic_auth(self._client_id, self._client_secret)}'
        }

        # Realizando a requisição
        resp = requests.post(
            ENDPOINT_ACCESS_TOKEN,
            data=data,
            timeout=DEFAULT_TIMEOUT_AUTHENTICATION,
            headers=headers
        )

        # Tratando o resultado
        if resp.status_code != 200:
            raise AuthenticationException(
                f'Erro de autenticação do usuário: {resp.text}')

        json_resp = resp.json()
        self._access_token = json_resp['access_token']
        self._token_type = json_resp['token_type']
        self._access_token_expires_in = json_resp['expires_in']
        self._access_token_scope = json_resp['scope']
        self._refresh_token = json_resp['refresh_token']

        # Retornando o acess_token
        return self._access_token

    def _get_basic_auth(self, client_id: str, client_secret: str):
        """
        Codifica a concatenação "<client_id>:<client_secret>" em base64, para autenticação basica,
        junto ao servidor de autenticação oauth (para recuperar um access_token).

        Parametros:
        - client_id: ID da aplicação cliente cadastrada no servidor de autenticação.
        - client_secret: Chave da aplicação cliente cadastrada no servidor de autenticação.
        """

        basic_credentials = f'{client_id}:{client_secret}'
        basic_credentials = base64.b64encode(
            basic_credentials.encode('utf-8')).decode('utf-8')

        return basic_credentials



GCLICK_URL_BASE = 'https://g2apiteste.gclick.com.br'
ENDPOINT_ATIVIDADES_RESPOSTA = GCLICK_URL_BASE + '/atividades/resposta'
GCLICK_URL_OAUTH_BASE = 'https://g2apiteste.gclick.com.br'
ENDPOINT_ACCESS_TOKEN = GCLICK_URL_OAUTH_BASE + '/oauth/token'
DEFAULT_TIMEOUT_AUTHENTICATION = 10
DEFAULT_TIMEOUT_UPLOAD = 60
